#include "Checker.hpp"

bool Checker::is_controlled(const Board& board, const Coord& coord, const Color& acolor) {
    const Color color = acolor.inverse();
    const Color oponent_color = color.inverse();

    // Directional attacks (Queen, Rook, Bishop, King)
    for (SNumber direction_file = -1; direction_file <= 1; ++direction_file) {
        for (SNumber direction_rank = -1; direction_rank <= 1; ++direction_rank) {
            for (SNumber delta = 1; delta <= 8; ++delta) {
                if (!Coord::is_valid(coord.file+delta*direction_file, coord.rank+delta*direction_rank)) break;
                auto to = Coord(coord.file+delta*direction_file, coord.rank+delta*direction_rank);

                if (!board.has_piece_at(to)) {
                    // not attacked
                } else {
                    auto attacking_piece = board.get_piece_at(to);
                    if (attacking_piece.color == oponent_color ) {
                        if ( attacking_piece.type == QUEEN )
                            return true;
                        if ( attacking_piece.type == BISHOP && ((direction_file == 0) == (direction_rank == 0)) )
                            return true;
                        if (attacking_piece.type == ROOK && ((direction_file == 0) || (direction_rank == 0)))
                            return true;
                        if (attacking_piece.type == KING && delta == 1)
                            return true;
                    }
                    break;
                }
            }
        }
    }

    // Pawn attacks
    SNumber direction_rank = (color == WHITE)? 1: -1;
    for (UNumber to_file = coord.file - 1; to_file <= coord.file + 1; to_file+=2 ) {
		if (!Coord::is_valid(to_file, coord.rank + direction_rank)) continue;
        const Coord to = Coord(to_file, coord.rank + direction_rank);

        if (board.has_piece_at(to) && board.get_piece_at(to) == Piece(oponent_color, PAWN))
            return true;
    }

    // Knight attacks
    for (SNumber delta_file = -2; delta_file <= 2; ++delta_file) {
        for (SNumber delta_rank = -2; delta_rank <= 2; ++delta_rank) {
            if (std::abs(delta_file) + std::abs(delta_rank) != 3) continue;
            if (!Coord::is_valid(coord.file + delta_file, coord.rank + delta_rank)) continue;

            const Coord to(coord.file + delta_file, coord.rank + delta_rank);

            if (!board.has_piece_at(to)) {
                // not attacked
            } else if (board.get_piece_at(to) == Piece(oponent_color, KNIGHT)) {
                return true;
            }
        }
    }

    return false;
}

bool Checker::is_attacked(const Board& board, const Coord& coord) {
    const Color color = board.get_piece_at(coord).color.inverse();

    return Checker::is_controlled(board, coord, color);
}

bool Checker::is_check(const Board& board, const Color& c) {
    if (c == NOT_A_COLOR) return Checker::is_check(board, board.turn_color);

    for (const auto& square: board.squares) {
        auto piece = square.second;

        if (piece == Piece(c, KING)) {
            const auto& king_position = square.first;

            return Checker::is_attacked(board, king_position);
        }
    }

    return false;
}
